"""

"""

import logging
import aiosqlite
from os.path import isfile
from sqlalchemy.ext.asyncio import AsyncSession, create_async_engine
from sqlalchemy.orm import sessionmaker

DB_PATH = "db/db.sqlite"
BUILD_PATH = "db/build.sql"
DATABASE_URL = "sqlite:///db/db.sqlite"
DATABASE_ASYNC_URL = "sqlite+aiosqlite:///db/db.sqlite"

log = logging.getLogger(__name__)


class DatabaseManager:
    def __init__(self, database_url=DATABASE_ASYNC_URL):
        self.engine = create_async_engine(database_url, future=True)
        self.session_maker = sessionmaker(self.engine, class_=AsyncSession)
        self.session = None

    async def __aenter__(self):
        self.session = self.session_maker()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.session.close()
        await self.session.commit()
        self.session = None
        await self.engine.dispose()


# class DBConnection:
#     """
#     Asynchronous context manager for database connections.
#     """

#     def __init__(self):
#         self.conn = None

#     async def __aenter__(self):
#         self.conn = await connect()
#         return self.conn

#     async def __aexit__(self, *args):
#         await close(self.conn)


# async def connect():
#     log.info("Opening database connection")
#     return await aiosqlite.connect(DB_PATH)

# async def with_commit(func):
#     """
#     Wrapper to commit changes to the database.
#     """
#     async def inner(*args, **kwargs):
#         await func(*args, **kwargs)
#         await commit()

#     return inner

# async def build(conn):
#     """
#     Build the database from the build script.
#     """
#     log.info("Building database from build script")

#     if isfile(BUILD_PATH):
#         await scriptexec(conn, BUILD_PATH)
#         return

#     raise ValueError('Build script not found')

# async def commit(conn):
#     """
#     Commit changes to the database.
#     """
#     log.info("Committing database changes")
#     await conn.commit()

# async def close(conn):
#     """
#     Close the database connection.
#     """
#     log.debug("Closing database connection")
#     await conn.close()

# async def field(conn, cmd, *vals):
#     """
#     Return a single field.
#     """
#     log.debug("Executing command for field: %s, vals:%s", cmd, vals)
#     async with conn.execute(cmd, tuple(vals)) as cur:
#         if (fetch := await cur.fetchone()) is not None:
#             return fetch[0]

# async def record(conn, cmd, *vals):
#     """
#     Return a single record.
#     """
#     log.debug("Executing command for record: %s, vals: %s", cmd, vals)
#     async with conn.execute(cmd, tuple(vals)) as cur:
#         return await cur.fetchone()

# async def records(conn, cmd, *vals):
#     """
#     Return all records.
#     """
#     log.debug("Executing command for records: %s, vals: %s", cmd, vals)
#     async with conn.execute(cmd, tuple(vals)) as cur:
#         return await cur.fetchall()

# async def column(conn, cmd, *vals):
#     """
#     Return a single column.
#     """
#     log.debug("Executing command for column: %s, vals: %s", cmd, vals)
#     async with conn.execute(cmd, tuple(vals)) as cur:
#         return [item[0] for item in await cur.fetchall()]

# async def execute(conn, cmd, *vals):
#     """
#     Execute a command.
#     """
#     log.debug("Executing command: %s, vals: %s", cmd, vals)
#     async with conn.execute(cmd, tuple(vals)) as cur:
#         return cur

# async def multiexec(conn, cmd, valset):
#     """
#     Execute multiple commands.
#     """
#     log.debug("Executing multiple commands: %s, valset: %s", cmd, valset)
#     async with conn.executemany(cmd, valset):
#         pass

# async def scriptexec(conn, path):
#     """
#     Execute a script.
#     """
#     log.debug("Executing script: %s", path)
#     with open(path, 'r', encoding='utf-8') as script:
#         await conn.executescript(script.read())
